@@ -7,8 +7,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXPrivate.h,v 1.1.2.3 2007/08/23 10:45:16 das Exp $
  */
 
 #ifndef _TKMACPRIV
@@ -36,6 +34,8 @@
     #define kThemeBackgroundMetal 6
     #define kThemeIncDecButtonSmall 21
     #define kThemeIncDecButtonMini 22
+    #define kThemeComboBox 16
+    #define kThemeMiniSystemFont 109
     #define kAppearancePartUpButton 20
     #define kAppearancePartDownButton 21
     #define kAppearancePartPageUpArea 22
@@ -62,7 +62,7 @@
     #else
     #define kCGBitmapByteOrder32Host (2 << 12)
     #endif
-    #endif
+#endif
 /* Define constants only available on Mac OS X 10.5 or later */
 #if MAC_OS_X_VERSION_MAX_ALLOWED < 1050
     #define kWindowUnifiedTitleAndToolbarAttribute (1L << 7)
@@ -76,7 +76,7 @@
     #define kHIToolboxVersionNumber10_4 (219)
 #endif
 #ifndef kHIToolboxVersionNumber10_5
-    #define kHIToolboxVersionNumber10_5 (316)
+    #define kHIToolboxVersionNumber10_5 (343)
 #endif
 /* Macros for HIToolbox runtime version checking */
 MODULE_SCOPE float tkMacOSXToolboxVersionNumber;
@@ -252,6 +252,7 @@ typedef struct TkMacOSXDrawingContext {
     CGrafPtr port, savePort;
     ThemeDrawingState saveState;
     RgnHandle saveClip;
+    HIShapeRef clipRgn;
     PixPatHandle penPat;
     Rect portBounds;
     Boolean portChanged;
@@ -264,6 +265,33 @@ typedef struct TkMacOSXDrawingContext {
 MODULE_SCOPE RgnHandle tkMacOSXtmpQdRgn;
 MODULE_SCOPE int tkMacOSXUseCGDrawing;
 
+/*
+ * Prototypes for TkMacOSXRegion.c.
+ */
+
+#if 0
+MODULE_SCOPE void TkMacOSXEmtpyRegion(TkRegion r);
+MODULE_SCOPE int TkMacOSXIsEmptyRegion(TkRegion r);
+#endif
+MODULE_SCOPE HIShapeRef TkMacOSXGetNativeRegion(TkRegion r);
+MODULE_SCOPE void TkMacOSXSetWithNativeRegion(TkRegion r, HIShapeRef rgn);
+MODULE_SCOPE void TkMacOSXOffsetRegion(TkRegion r, short dx, short dy);
+MODULE_SCOPE HIShapeRef TkMacOSXHIShapeCreateEmpty(void);
+MODULE_SCOPE HIMutableShapeRef TkMacOSXHIShapeCreateMutableWithRect(
+	const CGRect *inRect);
+MODULE_SCOPE OSStatus  TkMacOSXHIShapeSetWithShape(
+	HIMutableShapeRef inDestShape, HIShapeRef inSrcShape);
+#if 0
+MODULE_SCOPE OSStatus TkMacOSXHIShapeSetWithRect(HIMutableShapeRef inShape,
+	const CGRect *inRect);
+#endif
+MODULE_SCOPE OSStatus TkMacOSHIShapeDifferenceWithRect(
+	HIMutableShapeRef inShape, const CGRect *inRect);
+MODULE_SCOPE OSStatus TkMacOSHIShapeUnionWithRect(HIMutableShapeRef inShape,
+	const CGRect *inRect);
+MODULE_SCOPE OSStatus TkMacOSHIShapeUnion(HIShapeRef inShape1,
+	HIShapeRef inShape2, HIMutableShapeRef outResult);
+
 /*
  * Prototypes of TkAqua internal procs.
  */
@@ -285,8 +313,8 @@ MODULE_SCOPE int TkMacOSXIsWindowZoomed(TkWindow *winPtr);
 MODULE_SCOPE int TkGenerateButtonEventForXPointer(Window window);
 MODULE_SCOPE EventModifiers TkMacOSXModifierState(void);
 MODULE_SCOPE int TkMacOSXSetupDrawingContext(Drawable d, GC gc, int useCG,
-    TkMacOSXDrawingContext *dc);
-MODULE_SCOPE void TkMacOSXRestoreDrawingContext(TkMacOSXDrawingContext *dc);
+    TkMacOSXDrawingContext *dcPtr);
+MODULE_SCOPE void TkMacOSXRestoreDrawingContext(TkMacOSXDrawingContext *dcPtr);
 MODULE_SCOPE void TkMacOSXSetColorInPort(unsigned long pixel, int fg,
 	PixPatHandle penPat, CGrafPtr port);
 MODULE_SCOPE void TkMacOSXSetColorInContext(unsigned long pixel,
@@ -303,5 +331,8 @@ MODULE_SCOPE int TkMacOSXMakeFullscreen(TkWindow *winPtr, WindowRef window,
 MODULE_SCOPE void TkMacOSXEnterExitFullscreen(TkWindow *winPtr, int active);
 MODULE_SCOPE void TkMacOSXBringWindowForward(WindowRef wRef);
 MODULE_SCOPE WindowRef TkMacOSXDrawableWindow(Drawable drawable);
+MODULE_SCOPE void TkMacOSXWinCGBounds(TkWindow *winPtr, CGRect *bounds);
+MODULE_SCOPE HIShapeRef TkMacOSXGetClipRgn(Drawable drawable);
+MODULE_SCOPE Tcl_Obj* TkMacOSXGetStringObjFromCFString(CFStringRef str);
 
 #endif /* _TKMACPRIV */
